
MESSAGE:New( " ------ RIO Training Mission Script  r14 ------", 20):ToAll()
MESSAGE:New( "  --- DO NOT SWITCH OFF ACTIVE PAUSE ---", 20):ToAll()
MESSAGE:New( " --------- Press 2 to enter the RIO seat ---------", 20):ToAll()
MESSAGE:New( " ", 20):ToAll()

-- seed the random number generator
math.random()  
math.random() 
math.random()

--------------------------------------------------------------------------------------- FUNCTIONS ---

-- function to round a number
function round(num, numDecimalPlaces)
    local mult = 10^(numDecimalPlaces or 0)
    return math.floor(num * mult + 0.5) / mult
  end


-- getRandomSpeed
function getRandomSpeed()
    local i = math.random(20)
    speed = 250 + (i*15)
    return speed
end


-- Get a random plane
local function GetRandomPlane(planes)
    local N=#planes
    if N>0 then
      local i=math.random(N)
      local planeToReturn=planes[i]
      return planeToReturn
    else
      return nil
    end
end


-- Get a random zone
local function GetRandomZone(zones)
    local N=#zones
    if N>0 then
        local id=math.random(N)
        while id == spawnZoneID do
            id=math.random(N)
        end
        local zoneToReturn=zones[id]
        spawnZoneID = id
        return zoneToReturn
    else
        return nil
    end
end


-- get a height for the target aircraft
local function getRandomHeight()
    local i = math.random(100)
    height = 500 + (i*110)
    return height
end


-- find bogeyDirection to set off with
local function bogeyDirection(point1, point2)
    local deltaNorthing = point2.x - point1.x
    local deltaEasting = point2.z - point1.z
    local angle = (180/3.1416) * math.atan(deltaEasting / deltaNorthing)
    local bearing = round(angle,0)
    if deltaNorthing <=0 then
        bearing = bearing + 180
    end
    return bearing
end


-- Spawn a new unit
local function SpawnBogey()
    MESSAGE:New( " -- Bandit Spawned --", 10):ToAll()
    startPoint = spawnZone:GetCoordinate()
    endPoint = destinationZone:GetCoordinate()
    bogeyStartBearing = bogeyDirection(startPoint, endPoint)
    spawnHeight = getRandomHeight()

    -- AI plane constructor
    local planeToSpawn = GetRandomPlane(spawnPlanesTable)
    planeAI = SPAWN:New( planeToSpawn )
                :OnSpawnGroup(
                    function(group)
                        local route = {}                        
                        local speedKnots = getRandomSpeed()
                        local speedkph = speed * 1.852                       
                        local bogeyDestination = destinationZone:GetCoordinate()
                        local wpt2 = bogeyDestination:WaypointAir("BARO",COORDINATE.WaypointType.FlyoverPoint,nil,speedkph,true,nil,nil,"Waypoint")
                        wpt2.alt = spawnHeight
                        table.insert(route,wpt2)                                    
                        group:Route(route)
                        group:SetAltitude(spawnHeight, true, "BARO")
                    end
                )
                :InitHeading(bogeyStartBearing)
    -- spawn the plane - this is the UNIT
    planeAI_Spawn = planeAI:SpawnInZone(spawnZone, true, spawnHeight, spawnHeight)
                        

end


-- despawn the bogey as it has reached the destination zone
local function DespawnBogey()    
    MESSAGE:New( " -- Bandit Despawned --", 10):ToAll()
    planeAI_Spawn:Destroy()
end


-- respawn random bogey
local function RespawnBogey()
    DespawnBogey()
    spawnZone = GetRandomZone(spawnZones)
    destinationZone = GetRandomZone(destinationZones)
    spawnZoneID = 99
    SpawnBogey(spawnZone, destinationZone)
end

-- respawn random bogey Hot
local function RespawnBogeyHot()
    DespawnBogey()
    spawnZone = GetRandomZone(spawnZones)
    while spawnZoneID == 7 do
        spawnZone = GetRandomZone(spawnZones)
    end
    destinationZone = ZONE:New("ZNDW")
    spawnZoneID = 99
    SpawnBogey(spawnZone, destinationZone)
end

-- respawn random bogey Flanking
local function RespawnBogeyFlanking()
    DespawnBogey()
    spawnZone = GetRandomZone(spawnZones)
    while spawnZoneID == 7 do
        spawnZone = GetRandomZone(spawnZones)
    end
    if spawnZoneID == 1 or spawnZoneID == 2 then
        destinationZone = ZONE:New("ZNDSW")
    elseif spawnZoneID == 4 or spawnZoneID == 5 then
        destinationZone = ZONE:New("ZNDNW")
    elseif spawnZoneID == 8 or spawnZoneID == 6 then
        destinationZone = ZONE:New("ZNDW")
    elseif spawnZoneID == 3 or spawnZoneID == 9 then
        local zones = {ZONE:New("ZNDNW"), ZONE:New("ZNDSW")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    else 
        MESSAGE:New( " -- Issue with Flanking Bandit Spawn - Retry --", 10):ToAll()
    end
    spawnZoneID = 99
    SpawnBogey(spawnZone, destinationZone)
end

-- respawn random bogey Beaming
local function RespawnBogeyBeam()
    DespawnBogey()   
    spawnZone = GetRandomZone(spawnZones)
    while spawnZoneID == 7 do
        spawnZone = GetRandomZone(spawnZones)
    end
    spawnZone = GetRandomZone(spawnZones)
    if spawnZoneID == 1 then
        destinationZone = ZONE:New("ZNDS")
    elseif spawnZoneID == 5 then
        destinationZone = ZONE:New("ZNDN")
    elseif spawnZoneID == 3 then
        local zones = {ZONE:New("ZNDNE"), ZONE:New("ZNDSE")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 6 then
        local zones = {ZONE:New("ZNDNW"), ZONE:New("ZNDSW")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 2 or spawnZoneID == 4 or spawnZoneID == 9 then
        local zones = {ZONE:New("ZNDN"), ZONE:New("ZNDS")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 6 then
        destinationZone = ZONE:New("ZNDNW")
    elseif spawnZoneID == 8 then
        destinationZone = ZONE:New("ZNDSW")
    else 
        MESSAGE:New( " -- Issue with Beaming Bandit Spawn - Retry --", 10):ToAll()
    end
    spawnZoneID = 99
    SpawnBogey(spawnZone, destinationZone)
end


-- respawn random bogey Dragging
local function RespawnBogeyDrag()
    DespawnBogey()
    spawnZone = GetRandomZone(spawnZones)
    while spawnZoneID == 7 do
        spawnZone = GetRandomZone(spawnZones)
    end
    if spawnZoneID == 1 then
        destinationZone = ZONE:New("ZNDNE")
    elseif spawnZoneID == 2 then
        local zones = {ZONE:New("ZNDNE"), ZONE:New("ZNDE")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 3 then
            destinationZone = ZONE:New("ZNDE")
    elseif spawnZoneID == 4 then
        local zones = {ZONE:New("ZNDSE"), ZONE:New("ZNDE")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 5 then
        destinationZone = ZONE:New("ZNDSE")
    elseif spawnZoneID == 6 then
        local zones = {ZONE:New("ZNDNE"), ZONE:New("ZNDE"), ZONE:New("ZNDSE"), ZONE:New("ZNDS")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 7 then
        local zones = {ZONE:New("ZNDN"), ZONE:New("ZNDNE"), ZONE:New("ZNDE"), ZONE:New("ZNDSE"), ZONE:New("ZNDS")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 8 then
        local zones = {ZONE:New("ZNDN"), ZONE:New("ZNDNE"), ZONE:New("ZNDE"), ZONE:New("ZNDSE")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    elseif spawnZoneID == 9 then
        local zones = {ZONE:New("ZNDNE"), ZONE:New("ZNDE"), ZONE:New("ZNDSE")}
        local id=math.random(#zones)
        destinationZone = zones[id]
    else 
        MESSAGE:New( " -- Issue with Dragging Bandit Spawn - Retry --", 10):ToAll()
    end
    spawnZoneID = 99
    SpawnBogey(spawnZone, destinationZone)
end



-- find bogey altitude
local function bogeyAltitude()
    altitude = planeAI_Spawn:GetAltitude()
    altitudefeet = altitude * 3.28084
    altitudeRounded = round(altitudefeet, 0)
    MESSAGE:New("The bogey is at an altitude of: " .. altitudeRounded .. "ft", 20):ToAll()
end

-- find bogey speed
local function bogeySpeed()
    velocity = planeAI_Spawn:GetVelocityKNOTS()
    velocityRounded = round(velocity,0)
    MESSAGE:New("The bogey's speed is: " .. velocityRounded .. "knots", 20):ToAll()
end

-- find bogey distance
local function bogeyDistance()
    local bogeypos2 = planeAI_Spawn:GetVec2()
    local F14pos2   = playerGroup:GetPointVec2()
    local deltaX = bogeypos2.x - F14pos2.x
    local deltaY = bogeypos2.y - F14pos2.z
    local dist = math.sqrt((deltaX * deltaX) + (deltaY * deltaY))
    local distnm = round(dist/1852 , 0)
    MESSAGE:New("The bogey's range is: " .. distnm .. "nm", 20):ToAll()
end

-- find bogey bearing
local function bogeyBearing()
    local bogeypos2 = planeAI_Spawn:GetVec2()
    local F14pos2   = playerGroup:GetPointVec2()
    local deltaX = bogeypos2.x - F14pos2.x
    local deltaY = bogeypos2.y - F14pos2.z
    local angle = (180/3.1416) * math.atan(deltaX / deltaY)
    local bearing = round(90 - angle,0)
    MESSAGE:New("The bogey's bearing is: " .. bearing .. "deg", 20):ToAll()
end

-- find bogey heading
local function bogeyHeading()
    local heading = planeAI_Spawn:GetHeading()
    MESSAGE:New("The bogey's heading is: " .. heading .. "deg", 20):ToAll()
end


-- find bogey BRAA
local function bogeyBRAA()
    local bogeypos3 = planeAI_Spawn:GetCoordinate()
    local F14pos3   = playerGroup:GetCoordinate()
    local BRAA = bogeypos3:ToStringBRAANATO(F14pos3)
    MESSAGE:New(BRAA , 30):ToAll()
end

 -- menu function
local function menuForUser()

    local MenuReSpawn = MENU_COALITION:New (coalition.side.BLUE, "New Bogey Spawn...")
    local MenuRespawnRandom = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "New Bogey - Random", MenuReSpawn, RespawnBogey)
    local MenuRespawnRandom = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "New Bogey - Hot", MenuReSpawn, RespawnBogeyHot)
    local MenuRespawnRandom = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "New Bogey - Flanking", MenuReSpawn, RespawnBogeyFlanking)
    local MenuRespawnRandom = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "New Bogey - Beaming", MenuReSpawn, RespawnBogeyBeam)
    local MenuRespawnRandom = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "New Bogey - Dragging", MenuReSpawn, RespawnBogeyDrag)

    local menuTargetInfo = MENU_COALITION:New (coalition.side.BLUE, "Target Info...")
    local menuTargetAltitude = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey Altitude", menuTargetInfo, bogeyAltitude)
    local menuTargetSpeed = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey Speed", menuTargetInfo, bogeySpeed)
    local menuTargetRange = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey Range", menuTargetInfo, bogeyDistance)
    local menuTargetBearing = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey Bearing", menuTargetInfo, bogeyBearing)
    local menuTargetHeading = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey Heading", menuTargetInfo, bogeyHeading)
    local menuTargetBRAA = MENU_COALITION_COMMAND:New (coalition.side.BLUE, "Bogey BRAA", menuTargetInfo, bogeyBRAA)

end




--------------------------------------------------------------------------------------- TABLES ---

spawnPlanesTable = {
    "AR1",
    "AR2",
    "AR3",
    "AU1",
    "AU2" }

spawnZoneID = 99

spawnZones = {
        ZONE:New("ZNSN"),
        ZONE:New("ZNSNE"),
        ZONE:New("ZNSE"),
        ZONE:New("ZNSSE"),
        ZONE:New("ZNSS"),
        ZONE:New("ZNSSW"),
        ZONE:New("ZNSW"),
        ZONE:New("ZNSNW"),
        ZONE:New("ZNSC") }

destinationZones = {
    ZONE:New("ZNDN"),
    ZONE:New("ZNDNE"),
    ZONE:New("ZNDE"),
    ZONE:New("ZNDSE"),
    ZONE:New("ZNDS"),
    ZONE:New("ZNDSW"),
    ZONE:New("ZNDW"),
    ZONE:New("ZNDNW"),
    ZONE:New("ZNDC")}

playerGroup = GROUP:FindByName( "F14A" )




--------------------------------------------------------------------------------------- MAIN ---
spawnZone = GetRandomZone(spawnZones)
destinationZone = GetRandomZone(destinationZones)
SpawnBogey()
menuForUser()

SchedulerZoneCheck = SCHEDULER:New( nil, 
    function()
        if planeAI_Spawn:IsInZone(destinationZone) then
            MESSAGE:New( " *** Bandit arrived at destination ***", 10):ToAll()
            DespawnBogey()
            SpawnBogey()
        end
    end, {}, 10, 20 )



